"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupStreamApi = void 0;
const pump_1 = __importDefault(require("pump"));
const range_parser_1 = __importDefault(require("range-parser"));
const http_errors_1 = require("http-errors");
const helpers_1 = require("../helpers");
const validation_1 = require("../helpers/validation");
function setupStreamApi(app, config, _logger, client) {
    app.get('/stream', async (req, res) => {
        const encodeToken = config.security.streamApi.key || config.security.apiKey;
        const data = encodeToken
            ? helpers_1.verifyJwtToken(validation_1.validateString(req.query.token, 'token'), encodeToken, config.security.streamApi.maxAge)
            : req.query;
        if (!data) {
            throw new http_errors_1.Forbidden();
        }
        const link = validation_1.validateString(data.torrent, 'torrent');
        const fileName = validation_1.validateString(data.file, 'file');
        const torrent = await client.addTorrent(link);
        const file = torrent.files.find((f) => f.path === fileName) ||
            torrent.files.find((f) => f.name === fileName) ||
            torrent.files.find((f) => f.type.includes('video')) ||
            torrent.files[0];
        if (!file) {
            throw new http_errors_1.NotFound();
        }
        res.statusCode = 200;
        res.setHeader('Accept-Ranges', 'bytes');
        res.attachment(file.name);
        req.connection.setTimeout(3600000);
        const parsedRange = req.headers.range
            ? range_parser_1.default(file.length, req.headers.range)
            : undefined;
        const range = parsedRange instanceof Array ? parsedRange[0] : undefined;
        if (range) {
            res.statusCode = 206;
            res.setHeader('Content-Length', range.end - range.start + 1);
            res.setHeader('Content-Range', 'bytes ' + range.start + '-' + range.end + '/' + file.length);
        }
        else {
            res.setHeader('Content-Length', file.length);
        }
        if (req.method === 'HEAD') {
            return res.end();
        }
        return pump_1.default(file.createReadStream(range), res, () => {
            file.stop();
        });
    });
    return app;
}
exports.setupStreamApi = setupStreamApi;
