"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupTorrentsApi = void 0;
const http_errors_1 = require("http-errors");
const helpers_1 = require("../helpers");
const validation_1 = require("../helpers/validation");
function torrentToJson(v) {
    return {
        name: v.name,
        infoHash: v.infoHash,
        downloadSpeed: v.getDownloadSpeed(),
        downloaded: v.getDownloaded(),
        link: v.link,
        started: v.created.getTime(),
        updated: v.updated.getTime(),
        files: v.files.map((f) => ({
            name: f.name,
            path: f.path,
            type: f.type,
            length: f.length,
            stream: helpers_1.getSteamUrl(v.link, f.path),
        })),
    };
}
function setupTorrentsApi(app, _config, _logger, client) {
    app.post('/api/torrents', async (req, res) => {
        const link = validation_1.validateString(req.query.torrent, 'torrent');
        const torrent = await client.addTorrent(link);
        res.json(torrentToJson(torrent));
    });
    app.get('/api/torrents', (_req, res) => {
        return res.json(client.getTorrents().map(torrentToJson));
    });
    app.get('/api/torrents/:id', (req, res) => {
        const torrent = client.getTorrent(validation_1.validateString(req.params.id, 'id'));
        if (torrent) {
            return res.json(torrentToJson(torrent));
        }
        throw new http_errors_1.NotFound();
    });
    return app;
}
exports.setupTorrentsApi = setupTorrentsApi;
