"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readConfig = exports.isInHeroku = exports.isInGoogleAppEngine = void 0;
const fs_1 = require("fs");
const util_1 = require("util");
const helpers_1 = require("./helpers");
// Trust proxy by default if running in GoogleAppEngine
exports.isInGoogleAppEngine = process.env.GAE_APPLICATION ? true : false;
exports.isInHeroku = process.env._ ? process.env._.toLowerCase().includes('heroku') : false;
const defaultConfig = {
    host: process.env.HOST || '0.0.0.0',
    port: parseInt(process.env.PORT || '') || 3000,
    environment: process.env.NODE_ENV === 'development' ? 'development' : 'production',
    trustProxy: (process.env.TRUST_PROXY || '').toLowerCase() === 'true'
        ? true
        : exports.isInGoogleAppEngine || exports.isInHeroku,
    logging: {
        transports: [
            {
                type: 'console',
            },
        ],
        level: 'info',
    },
    torrents: {
        path: '/tmp/torrent-stream-server',
        autocleanInternal: 60 * 60,
    },
    security: {
        streamApi: {
            maxAge: '6h',
        },
        apiKey: undefined,
        frontendEnabled: true,
        apiEnabled: true,
    },
};
async function readConfig(path) {
    try {
        return path
            ? helpers_1.merge(defaultConfig, JSON.parse(await util_1.promisify(fs_1.readFile)(path, { encoding: 'utf8' })))
            : defaultConfig;
    }
    catch (error) {
        throw Error(`Failed to read config from ${path} - ${error}`);
    }
}
exports.readConfig = readConfig;
