"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebtorrentTorrentAdapter = void 0;
const _1 = require(".");
const webtorrent_1 = __importDefault(require("webtorrent"));
const util_1 = require("util");
const fs_1 = require("fs");
class WebtorrentTorrentAdapter extends _1.TorrentAdapter {
    constructor(config) {
        super(config);
        this.client = new webtorrent_1.default();
    }
    async add(magnet) {
        await util_1.promisify(fs_1.mkdir)(this.config.path, { recursive: true });
        const torrent = this.client.add(magnet, {
            path: this.config.path,
        });
        return new Promise((resolve, reject) => {
            torrent.on('error', (e) => {
                if (String(e).includes('duplicate torrent')) {
                    const value = this.client.get(torrent.infoHash);
                    if (value) {
                        resolve(this.createTorrentFromClient(value));
                    }
                    else {
                        reject(e);
                    }
                }
                else {
                    reject(e);
                }
            });
            torrent.on('metadata', () => {
                torrent.deselect(0, torrent.pieces.length - 1, 0);
                torrent.files.forEach((file) => {
                    file.deselect();
                });
            });
            torrent.on('ready', () => {
                resolve(this.createTorrentFromClient(torrent));
            });
        });
    }
    createTorrentFromClient(torrent) {
        return {
            name: torrent.name,
            files: torrent.files.map((file) => ({
                name: file.name,
                path: file.path,
                length: file.length,
                createReadStream(range) {
                    return file.createReadStream(range);
                },
                stop() {
                    file.deselect();
                },
            })),
            async remove() {
                new Promise((resolve) => {
                    torrent.destroy({
                        destroyStore: true,
                    }, () => {
                        resolve();
                    });
                });
            },
            getDownloadSpeed() {
                return torrent.downloadSpeed;
            },
            getDownloaded() {
                return torrent.downloaded;
            },
            getUploadSpeed() {
                return torrent.uploadSpeed;
            },
            getUploaded() {
                return torrent.uploaded;
            },
        };
    }
}
exports.WebtorrentTorrentAdapter = WebtorrentTorrentAdapter;
