"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TorrentClient = void 0;
const util_1 = require("util");
const parse_torrent_1 = __importDefault(require("parse-torrent"));
const mime_types_1 = require("mime-types");
const http_errors_1 = require("http-errors");
__exportStar(require("./adapters"), exports);
__exportStar(require("./adapters/webtorrent"), exports);
class TorrentClient {
    constructor(config, adapter, logger) {
        this.config = config;
        this.adapter = adapter;
        this.logger = logger;
        this.torrents = {};
        this.cleanLocked = false;
    }
    getTorrents() {
        return Object.values(this.torrents);
    }
    getTorrent(infoHash) {
        return this.torrents[infoHash];
    }
    async removeTorrent(infoHash) {
        const torrent = this.torrents[infoHash];
        if (torrent) {
            await torrent.remove();
            delete this.torrents[infoHash];
        }
    }
    async addTorrent(link) {
        let parsedLink;
        try {
            parsedLink = await util_1.promisify(parse_torrent_1.default.remote)(link);
        }
        catch (err) {
            throw new http_errors_1.BadRequest(`Cannot parse torrent: ${err instanceof Error ? err.message : err}, link: ${link}`);
        }
        if (!parsedLink) {
            throw new http_errors_1.BadRequest(`Cannot parse torrent: ${link}`);
        }
        const magnet = parse_torrent_1.default.toMagnetURI(parsedLink);
        const infoHash = parsedLink.infoHash;
        if (infoHash in this.torrents) {
            this.torrents[infoHash] = {
                ...this.torrents[infoHash],
                updated: new Date(),
            };
            return this.torrents[infoHash];
        }
        const torrent = await this.adapter.add(magnet).then((v) => ({
            ...v,
            link,
            infoHash,
            created: new Date(),
            updated: new Date(),
            files: v.files.map((f) => ({
                ...f,
                type: mime_types_1.lookup(f.name) || '',
            })),
        }));
        this.torrents[torrent.infoHash] = torrent;
        setTimeout(() => {
            this.checkForExpiredTorrents().catch((err) => {
                this.logger.error(err);
            });
        }, 1000);
        return torrent;
    }
    async checkForExpiredTorrents() {
        if (this.cleanLocked) {
            return;
        }
        this.cleanLocked = true;
        try {
            const torrentToRemove = Object.values(this.torrents).filter((torrent) => Date.now() - torrent.updated.getTime() > this.config.autocleanInternal * 1000);
            for (const torrent of torrentToRemove) {
                this.logger.info(`Removing expired ${torrent.name} torrent`);
                await this.removeTorrent(torrent.infoHash);
            }
        }
        finally {
            this.cleanLocked = false;
        }
    }
}
exports.TorrentClient = TorrentClient;
